#ifndef NOISEOCCUPANCY_H
#define NOISEOCCUPANCY_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/Sct_CalibrationController.hh"
#include "../CalibrationControllerImpl.h"

#include "../SctApiException.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {
    
class NoiseOccupancyTest : public TestRequestImpl {
public:	
    static NoiseOccupancyTest& instance() {
	static NoiseOccupancyTest no;
	return no;
    }
    
    virtual void startTest(Sct_SctApi_T_SctApi& api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = &api;
	
	Sct_SctApiStatus status;
	api.modifyABCDVar(&status, ST_COMPRESSION, 1);     // X1X
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "NoiseOccupancyTest::startTest", __FILE__, __LINE__);
	
	api.modifyABCDVar(&status, ST_EDGE_DETECT, 0);     // EDGE detect off.
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "NoiseOccupancyTest::startTest", __FILE__, __LINE__);
	
	api.modifyABCDVar(&status, ST_SEND_MASK, 0);     // Send mask off
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "NoiseOccupancyTest::startTest", __FILE__, __LINE__);

	name = "NoiseOccupancyTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	testPoints.push_back(0.);
    }
    
    virtual Sct_SctApi_T_Scan* getNextScan(unsigned long index) {
	if (api == 0) 
	    throw InvariantViolatedError("Api pointer is 0 in getNextScan!", __FILE__, __LINE__);
	
	ScanLibrary& sl = CalibrationControllerImpl::instance().getScanLibrary();
	
	Sct_SctApiStatus status;
	api->modifyABCDVar(&status, ST_QCAL, 0.);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "NoiseOccupancyTest::getNextScan", __FILE__, __LINE__);
	
	return &sl.thresholdScan(2000);
    }
    
    virtual bool canFeedback() {
	return false;
    }
    
    virtual void endTest() {
	api = 0;
    }
    
    
private:
    NoiseOccupancyTest() {}
    Sct_SctApi_T_SctApi* api;    
};
}
}
#endif //RESPONSECURVE_H
