#include "Service.h"
#include "ArchivingWorkerGroup.h"
#include "archiving/ArchRawScanResult.h"
#include "archiving/ArchFitScanResult.h"

#include "Sct/SctNames.h"
#include "is/isinfo.h" 
#include "Sct/IS/IOManagerIS.h"
#include "SctData/TestResult.h"
#include "SctData/RawScanResult.h"
#include "SctData/FitScanResult.h"
#include <string>

using namespace Sct;
using namespace SctData;
using namespace Sct::IS;
using namespace boost;

namespace SctArchivingService {

ArchivingService::ArchivingService() : IPCObject(Archiving_C_ArchivingService_interfaceName, &server()){
    group = new ArchivingWorkerGroup();
}


IPCServer& ArchivingService::server() {
    static IPCServer* server = new IPCServer(Archiving_C_ArchivingService_serverName, SctNames::getPartition());
    return *server;
}

void ArchivingService::run() {
    cout<<" I am in ArchivingService::run() " <<endl;

    IPCPartition& p = SctNames::getPartition();
    ISInfoReceiver r(p);

    group->go(1);
    
    r.subscribe("TestData", ".*TestResult.*", MyISCallback,this);
    r.run();
    
}


void ArchivingService::MyISCallback(ISCallbackInfo * isc) {
    cout<<" I am in ArchivingService::MyISCallback " <<endl;
    ArchivingService* a = static_cast<ArchivingService*>(isc->parameter());
    cout<< " !!!! IS Object Name = " << isc->type().name()<<endl;
    a->archive(isc);
}


void ArchivingService::archive(ISCallbackInfo * isc) {
    cout<<" I am in ArchivingService::archive " <<endl;
    
    if (isc->reason() != ISInfoCreated && isc->reason() != ISInfoUpdated ) return;

   
    shared_ptr<Serializable> ob = IOManagerIS::instance().read(*isc);
    cout << "!!!!!!!!! CLASS NAME ==== " << ob->getClassName() <<endl;
    if (ob == 0) {
        cerr << "Failed to get object from IS" << endl;
        return;
    }

    group->work(ob);
    cout << "Bye bye Test" <<endl;
    
}

ilu_T_CString ArchivingService::ping (ArchivingStatus *_status) {
    char* msg = new char[28];
    strcpy(msg, "Hello from ArchivingService");
    return msg;
}

}
